@@ -1,22 +1,35 @@
 from datasette import hookimpl
 from datasette.utils import path_with_replaced_args
+from datasette.utils.asgi import Response
+import csv
+import io
 import tabulate
 
 
-async def render_copyable(rows, request, datasette, database, table, query_name):
+async def render_copyable(
+    rows, columns, request, datasette, database, table, query_name
+):
     table_format = request.args.get("_table_format")
     raw = request.args.get("_raw")
 
     if table_format not in tabulate.tabulate_formats:
         table_format = "tsv"
 
-    copyable = tabulate.tabulate(rows, tablefmt=table_format)
+    if table_format == "tsv":
+        writer = io.StringIO()
+        csv_writer = csv.writer(writer, delimiter="\t")
+        csv_writer.writerow(columns)
+        for row in rows:
+            csv_writer.writerow(row)
+        copyable = writer.getvalue()
+    else:
+        copyable = tabulate.tabulate(rows, tablefmt=table_format, headers=columns)
 
     if raw:
         return {"body": copyable, "content_type": "text/plain; charset=utf-8"}
 
-    return {
-        "body": await datasette.render_template(
+    return Response(
+        await datasette.render_template(
             "copyable.html",
             {
                 "database": database,
@@ -36,8 +49,8 @@ async def render_copyable(rows, request, datasette, database, table, query_name)
             },
             request=request,
         ),
-        "content_type": "text/html; charset=utf-8",
-    }
+        content_type="text/html; charset=utf-8",
+    )
 
 
 @hookimpl
